@@ -18,12 +18,7 @@ setClass("SQLiteResult",
   slots = list(
     sql = "character",
     ptr = "externalptr",
-    conn = "SQLiteConnection"
+    conn = "SQLiteConnection",
+    bigint = "character"
   )
 )
-
-#' @rdname SQLiteResult-class
-#' @export
-setMethod("dbIsValid", "SQLiteResult", function(dbObj, ...) {
-  result_valid(dbObj@ptr)
-})
