@@ -1,49 +1,12 @@
-context("encoding")
-
-test_that("write tables whose colnames or contents are BIG5 encoded (#277)", {
-  skip_on_os("linux")
-  skip_on_os("mac")
-  skip_on_os("solaris")
-  .loc <- Sys.getlocale("LC_COLLATE")
-  Sys.setlocale(locale = "cht")
-  con <- dbConnect(SQLite())
-  on.exit({
-    Sys.setlocale(locale = .loc)
-    dbDisconnect(con)
-  })
-
-  big5_string <- rawToChar(as.raw(c(0xa4, 0xa4, 0xa4, 0xe5)))
-  df <- structure(
-    list(V1 = 1:3),
-    class = "data.frame",
-    row.names = 1:3
-  )
-  colnames(df) <- big5_string
-  dbWriteTable(con, "a", df)
-  res <- dbReadTable(con, "a")
-  expect_identical(res, df)
-
-  df <- structure(
-    list(V1 = 1:3, V2 = rep(big5_string, 3)),
-    class = "data.frame",
-    row.names = 1:3
-  )
-  colnames(df) <- paste(big5_string, 1:2, sep = "")
-  dbWriteTable(con, "b", df)
-  res <- dbReadTable(con, "b")
-  expect_identical(res, df)
-})
-
 test_that("write tables whose colnames and contents are UTF-8 encoded (#277)", {
-  .loc <- Sys.getlocale("LC_COLLATE")
   if (.Platform$OS.type == "windows") {
-    Sys.setlocale(locale = "cht")
+    skip_if_not(getRversion() >= "4.0")
+    withr::local_collate("cht")
   } else {
-    Sys.setlocale(locale = "zh_TW.UTF-8")
+    withr::local_collate("zh_TW.UTF-8")
   }
   con <- dbConnect(SQLite())
-  on.exit({
-    Sys.setlocale(locale = .loc)
+  withr::defer({
     dbDisconnect(con)
   })
 
@@ -74,15 +37,13 @@ test_that("list the field of tables whose colnames are BIG5 encoded (#277)", {
   skip_on_os("linux")
   skip_on_os("mac")
   skip_on_os("solaris")
-  .loc <- Sys.getlocale("LC_COLLATE")
   if (.Platform$OS.type == "windows") {
-    Sys.setlocale(locale = "cht")
+    withr::local_collate("cht")
   } else {
-    Sys.setlocale(locale = "zh_TW.UTF-8")
+    withr::local_collate("zh_TW.UTF-8")
   }
   con <- dbConnect(SQLite())
-  on.exit({
-    Sys.setlocale(locale = .loc)
+  withr::defer({
     dbDisconnect(con)
   })
 
@@ -92,6 +53,9 @@ test_that("list the field of tables whose colnames are BIG5 encoded (#277)", {
     class = "data.frame",
     row.names = c(NA, -3L)
   )
+
+  skip_on_os("windows")
+
   colnames(df) <- big5_string
   dbWriteTable(con, "a", df)
   expect_identical(dbListFields(con, "a"), colnames(df))
@@ -106,15 +70,13 @@ test_that("list the field of tables whose colnames are BIG5 encoded (#277)", {
 })
 
 test_that("list the field of tables whose colnames are UTF-8 encoded (#277)", {
-  .loc <- Sys.getlocale("LC_COLLATE")
   if (.Platform$OS.type == "windows") {
-    Sys.setlocale(locale = "cht")
+    withr::local_collate("cht")
   } else {
-    Sys.setlocale(locale = "zh_TW.UTF-8")
+    withr::local_collate("zh_TW.UTF-8")
   }
   con <- dbConnect(SQLite())
-  on.exit({
-    Sys.setlocale(locale = .loc)
+  withr::defer({
     dbDisconnect(con)
   })
 
@@ -139,15 +101,13 @@ test_that("list the field of tables whose colnames are UTF-8 encoded (#277)", {
 })
 
 test_that("append tables whose colnames are UTF-8 encoded (#277)", {
-  .loc <- Sys.getlocale("LC_COLLATE")
   if (.Platform$OS.type == "windows") {
-    Sys.setlocale(locale = "cht")
+    withr::local_collate("cht")
   } else {
-    Sys.setlocale(locale = "zh_TW.UTF-8")
+    withr::local_collate("zh_TW.UTF-8")
   }
   con <- dbConnect(SQLite())
-  on.exit({
-    Sys.setlocale(locale = .loc)
+  withr::defer({
     dbDisconnect(con)
   })
 
@@ -171,3 +131,41 @@ test_that("append tables whose colnames are UTF-8 encoded (#277)", {
   dbWriteTable(con, "b", df)
   expect_error(dbWriteTable(con, "b", df, append = TRUE), NA)
 })
+
+test_that("write tables whose colnames or contents are BIG5 encoded (#277)", {
+  skip_on_os("linux")
+  skip_on_os("mac")
+  skip_on_os("solaris")
+
+  .loc <- Sys.getlocale("LC_COLLATE")
+  suppressWarnings(Sys.setlocale(locale = "cht"))
+  withr::defer({
+    Sys.setlocale(locale = .loc)
+  })
+
+  con <- dbConnect(SQLite())
+  withr::defer({
+    dbDisconnect(con)
+  })
+
+  big5_string <- rawToChar(as.raw(c(0xa4, 0xa4, 0xa4, 0xe5)))
+  df <- structure(
+    list(V1 = 1:3),
+    class = "data.frame",
+    row.names = 1:3
+  )
+  colnames(df) <- big5_string
+  dbWriteTable(con, "a", df)
+  res <- dbReadTable(con, "a")
+  expect_identical(res, df)
+
+  df <- structure(
+    list(V1 = 1:3, V2 = rep(big5_string, 3)),
+    class = "data.frame",
+    row.names = 1:3
+  )
+  colnames(df) <- paste(big5_string, 1:2, sep = "")
+  dbWriteTable(con, "b", df)
+  res <- dbReadTable(con, "b")
+  expect_identical(res, df)
+})
